from datetime import date


__banner__ = r"""
               ___     
          ____|_  ___ _
         / __/ __/ _ `/
        /_/ /____\_, / 
 Reads to Genes /___/  
"""

__version__ = "0.1.1"

__title__ = "r2g_gui"
__full_name__ = "Reads to Genes (GUI)"
__url__ = "https://github.com/yangwu91/r2g_gui"

__author__ = "Yang Wu"
__maintainer__ = "Yang Wu"
__author_email__ = "wuyang@drwu.ga"
__license__ = "MIT Licence"
__copyright__ = "Copyright {} Yang Wu".format(date.today().year)
